#version 450
/*
 * Copyright 2021 TU Wien, Institute of Visual Computing & Human-Centered Technology.
 * This file is part of the ECG Lab Framework and must not be redistributed.
 *
 * Original version created by Lukas Gersthofer and Bernhard Steiner.
 * Vulkan edition created by Johannes Unterguggenberger (junt@cg.tuwien.ac.at).
 */

layout (location = 0) in vec3 in_position;

layout (binding = 0) uniform UniformBuffer {
	vec4 color;
	mat4 modelMatrix;
	mat4 modelMatrixForNormals;
	mat4 viewProjMatrix;
	vec4 cameraPosition;
	vec4 illumination; // ka, kd, ks, alpha
	float brightness;
	float time;
} ub_data;

layout (location = 0) out vec3 outUVW;


void main() {
	outUVW = in_position;
	// Convert cubemap coordinates into Vulkan coordinate space
	outUVW.xy *= -1.0;
	
	vec4 position = ub_data.viewProjMatrix * ub_data.modelMatrix * vec4(in_position.xyz, 1.0);
	// Set z more or less equal to w for almost maximum depth 
	// This is necessary to avoid z-fighting with the skybox
	position.z = position.w - 1e-6;
	gl_Position = position.xyzw;
}

