#version 450

layout (location = 0) in vec3 in_position;
layout (location = 1) in vec3 in_normal;
layout (location = 2) in vec2 in_texture_coordinates;

layout (binding = 0) uniform UniformBuffer {
	vec4 color;
	mat4 modelMatrix;
	mat4 modelMatrixForNormals;
	mat4 viewProjMatrix;
	vec4 cameraPosition;
	vec4 illumination; // ka, kd, ks, alpha
	float brightness;
	float time;
} ub_data;

layout (binding = 1) uniform DirectionalLight {
	vec4 color;
	vec4 direction;
	mat4 depthMVP;

} dl_data;

layout (binding = 2) uniform PointLight {
	vec4 color;
	vec4 position;
	vec4 attenuation;
} pl_data;

layout (location = 0) out VertexData {
	vec3 positionWorld;
	vec3 normalWorld;
	vec2 textureCoordinates;
	// Shadowmapping
	vec4 shadowCoordinates;
} vert_out;

// bias matrix for Shadowmapping
// transforms from [-1,1] to [0,1]
const mat4 biasMat = mat4(
	0.5, 0.0, 0.0, 0.0,
	0.0, 0.5, 0.0, 0.0,
	0.0, 0.0, 1.0, 0.0,
	0.5, 0.5, 0.0, 1.0 );

void main() {
	vec4 position_world = ub_data.modelMatrix * vec4(in_position, 1);
	vert_out.positionWorld = position_world.xyz;
	gl_Position = ub_data.viewProjMatrix * position_world;
	
	vert_out.normalWorld = mat3(ub_data.modelMatrixForNormals) * in_normal;
	vert_out.textureCoordinates = in_texture_coordinates;
	
	vert_out.shadowCoordinates = ( biasMat * dl_data.depthMVP * ub_data.modelMatrix ) * vec4(in_position, 1.0);
}
